@@ -5,7 +5,7 @@ import (
 	"os"
 	"syscall"
 
-	"github.com/containers/libpod/v2/libpod/define"
+	"github.com/containers/podman/v2/libpod/define"
 	"github.com/pkg/errors"
 )
 
