@@ -4,7 +4,8 @@ import (
 	"os"
 	"sync"
 
-	"github.com/containers/libpod/v2/pkg/domain/entities"
+	"github.com/containers/podman/v2/pkg/domain/entities"
+	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
 )
 
@@ -23,7 +24,15 @@ func IsRemote() bool {
 		fs.Usage = func() {}
 		fs.SetInterspersed(false)
 		fs.BoolVarP(&remoteFromCLI.Value, "remote", "r", false, "")
-		_ = fs.Parse(os.Args[1:])
+
+		// The shell completion logic will call a command called "__complete" or "__completeNoDesc"
+		// This command will always be the second argument
+		// To still parse --remote correctly in this case we have to set args offset to two in this case
+		start := 1
+		if len(os.Args) > 1 && (os.Args[1] == cobra.ShellCompRequestCmd || os.Args[1] == cobra.ShellCompNoDescRequestCmd) {
+			start = 2
+		}
+		_ = fs.Parse(os.Args[start:])
 	})
 	return podmanOptions.EngineMode == entities.TunnelMode || remoteFromCLI.Value
 }
