@@ -1,7 +1,7 @@
 package compat
 
 import (
-	"github.com/containers/libpod/v2/pkg/domain/entities"
+	"github.com/containers/podman/v2/pkg/domain/entities"
 	"github.com/containers/storage/pkg/archive"
 	"github.com/docker/docker/api/types"
 )
@@ -63,3 +63,17 @@ type swagCompatNetworkCreateResponse struct {
 	// in:body
 	Body struct{ types.NetworkCreate }
 }
+
+// Network disconnect
+// swagger:model NetworkConnectRequest
+type swagCompatNetworkConnectRequest struct {
+	// in:body
+	Body struct{ types.NetworkConnect }
+}
+
+// Network disconnect
+// swagger:model NetworkDisconnectRequest
+type swagCompatNetworkDisconnectRequest struct {
+	// in:body
+	Body struct{ types.NetworkDisconnect }
+}
