@@ -3,7 +3,7 @@ package libpod
 import (
 	"net/http"
 
-	"github.com/containers/libpod/v2/pkg/api/handlers/utils"
+	"github.com/containers/podman/v2/pkg/api/handlers/utils"
 	"github.com/pkg/errors"
 )
 
