@@ -3,7 +3,7 @@ package server
 import (
 	"net/http"
 
-	"github.com/containers/libpod/v2/pkg/api/handlers/compat"
+	"github.com/containers/podman/v2/pkg/api/handlers/compat"
 	"github.com/gorilla/mux"
 )
 
