@@ -3,8 +3,8 @@ package test_bindings
 import (
 	"time"
 
-	"github.com/containers/libpod/v2/pkg/bindings/containers"
-	"github.com/containers/libpod/v2/pkg/specgen"
+	"github.com/containers/podman/v2/pkg/bindings/containers"
+	"github.com/containers/podman/v2/pkg/specgen"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	"github.com/onsi/gomega/gexec"
@@ -35,7 +35,7 @@ var _ = Describe("Create containers ", func() {
 		s.Command = []string{"top"}
 		s.Terminal = true
 		s.Name = "top"
-		ctr, err := containers.CreateWithSpec(bt.conn, s)
+		ctr, err := containers.CreateWithSpec(bt.conn, s, nil)
 		Expect(err).To(BeNil())
 		data, err := containers.Inspect(bt.conn, ctr.ID, nil)
 		Expect(err).To(BeNil())
